C++ Library for accessing PCSC-lite, OpenSSL, PKCS#11
=====================================================

The Libraries for accessing smartcard tokens are written in ugly C, moving around pointers with no memory management. This library provides a simple and nice C++ wrapper around these libraries, so that programmers can concentrate on functionality. It offers general support for PCSC-lite, OpenSSL, PKCS#11, plus specific functionality for the SuisseID.


Modules
-------

**Note:** All *links* below require the generated [Doxygen](https://doc.mrw.sh/proxyface) documentation.

There are several [Namespaces](namespaces.html) which correspond to the [Modules](modules.html) that are implemented. All libraries libraries deal with hardware token cryptography. Some libraries are just clean C++-wrappers around the original libraries that are implemented in ugliest C manner. The warppers care about memory- and resource-management and implement a simple and easy C++ interface, including std::string for binary data and exceptions for error handling.

For special documentations, such as global overviews and tutorials, please refere to [Pages](pages.html).

The library contains the following [modules](modules.html):
 - [Siemens CardOS 4.4](group__gcardos.html):  
   APDU access to Siemens CardOS 4.4.  
   CardOS is an operating system on Siemens smart cards.
 - [Cryptoki API / PKCS#11](group__gcryptoki.html):  
   C++ wrapper around higher level PKCS#11 smart card access.  
   Cryptoki, also known as PKCS#11 is a higher level API.
 - [OpenSSL](group__gopenssl.html):  
   C++ wrapper around some OpenSSL functionality.  
   OpenSSL is a high level cryptography library.
 - [PCSC-Lite](group__gpcsc.html):  
   C++ wrapper around PCSC-Lite smart card reader access library.  
   PCSC-Lite is a middleware to access a smart card using SCard API.
 - [SuisseID](group__gsuisseid.html):  
   High level functions for the SuisseID hardware token.  
   SuisseID is a standardized digital identity in Switzerland.
 - [Crypto](group__gcrypto,html):  
   Crypto implements some auxiliary crypto funtions.

See [Overview of the Components](mainoverview.html) to get a graphical overview on the module structure and the hardware interaction.


In the Web
----------

 - [Official project page](https://mrw.sh/libraries/proxyface)
 - [Full Doxygen library documentation](https://doc.mrw.sh/proxyface)
 - [Official repositories](https://repository.mrw.sh)
 - [Download and installation instructions](https://mrw.sh/doc/mrw.sh/src/branch/master/installation.md)


Missing a Feature, Found a Bug
------------------------------

You are missing a feature, or an implementation is too incomplete for the purpose you need it? Or you even found a bug? Just register and open an issue [on the project management page](https://mrw.sh/libraries/libpcscxx/issues).
